#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "../../../../../Include/Utility/ResourceMgr/string_limit.h"
#define MAX_CHARS_PER_LINE	4096
#define MAX_LANGUAGES		32
#define MAX_ENTRIES
#define ENUM_HEADER	"str_table.h"
#define LIST_HEADER "id_list.txt"

int main(int argc, char *argv[]) {
	int i;
	char buffer[MAX_CHARS_PER_LINE];
	char buffer_2[MAX_CHARS_PER_LINE];

	char LANGUAGES[MAX_LANGUAGES][16];
	int language_id = 0;

	FILE *fp;
	char *tab_idx;
	char *buffer_idx;

	FILE *out_fp[MAX_LANGUAGES][2];
	FILE *header_fp[2];

	FILE *list_fp[2];

	char filename[256];

	size_t maxStringLength;
	int shortCnt = 0;
	int longCnt = 0;

	if(argc != 2) {
		printf("Realtek Venus DVR Supplement Tool\n");
		printf("\t\t2005/06/27 Chih-pin Wu <wucp@realtek.com.tw>\n");
		printf("Usage: convert <unicode text file output by excel>\n");
		return 0;
	}

	fp = fopen(argv[1], "r");

	if(fp == NULL) {
		printf("fatal error: unable to open file [%s]\n", argv[1]);
		return 0;
	}

	// parsing first lines for language definitions
	fgets(buffer, MAX_CHARS_PER_LINE, fp);
	buffer_idx = &(buffer[1]);

	while(((tab_idx = strchr(buffer_idx, '\t')) != NULL)||((tab_idx = strchr(buffer_idx, 0x0A)) != NULL)) {
		strncpy(LANGUAGES[language_id], buffer_idx, tab_idx - buffer_idx);
		LANGUAGES[language_id][tab_idx-buffer_idx] = '\0';
		language_id++;
		buffer_idx += ((tab_idx - buffer_idx) +1);
	}

	// open file pointers for files [language].str  
	for(i=0;i<language_id;i++) {

		sprintf(filename, "%s.str", LANGUAGES[i]);
		if((out_fp[i][0] = fopen(filename, "w")) == NULL) {
			printf("fopen(%s) failed.\n", filename);
			assert(0);
		}

		sprintf(filename, "%s.tmp", LANGUAGES[i]);
		if((out_fp[i][1] = fopen(filename, "w+")) == NULL) {
			printf("fopen(%s) failed.\n", filename);
			assert(0);
		}
	}

	// header file
	header_fp[0] = fopen(ENUM_HEADER, "w");
	if(header_fp[0] == NULL)
		assert(0);

	sprintf(filename, "%s.tmp", ENUM_HEADER);
	header_fp[1] = fopen(filename, "w+");
	if(header_fp[1] == NULL)
		assert(0);

	// string id list file
	list_fp[0] = fopen(LIST_HEADER, "w");
	if(list_fp[0] == NULL)
		assert(0);

	sprintf(filename, "%s.tmp", LIST_HEADER);
	list_fp[1] = fopen(filename, "w+");
	if(list_fp[1] == NULL)
		assert(0);

	// header file output for "language enumerations"
	fprintf(header_fp[0], "#ifndef __STRING_TABLE_HEADER__\n");
	fprintf(header_fp[0], "#define __STRING_TABLE_HEADER__\n");
	fprintf(header_fp[0], "/*\nLanguage supported\n*/\n");
	fprintf(header_fp[0], "typedef enum {\n");

	for(i = 0 ; i < language_id ; i++) {
		fprintf(header_fp[0], "\tLANG_%s,\n", LANGUAGES[i]);
	}

	fprintf(header_fp[0], "} ENUM_LANGUAGE;\n\n");
	fprintf(header_fp[0], "/*\nString definition\n*/\n");
	fprintf(header_fp[0], "typedef enum {\n");

	// one empty line
	fgets(buffer, MAX_CHARS_PER_LINE, fp);

	// start parsing
	while(feof(fp) == 0) {
		int fileidx;

		fgets(buffer, MAX_CHARS_PER_LINE, fp);
		if(strncmp(buffer, "STRING_NUM", strlen("STRING_NUM"))== 0)
			break;

		// first pass - decide this string is long or short
		maxStringLength = 0;
		buffer_idx = &(buffer[0]);

		for(i = 0 ; i < (language_id+1) ; i++) {
			tab_idx = strchr(buffer_idx, '\t');
			if(tab_idx == NULL && i == language_id) {
				if(strlen(buffer_idx) > maxStringLength)
					maxStringLength = strlen(buffer_idx);
			}
			else {
				strncpy(buffer_2, buffer_idx, tab_idx-buffer_idx);
				buffer_2[tab_idx-buffer_idx] = '\0';
				buffer_idx += ((tab_idx - buffer_idx) +1);

				if(strlen(buffer_2) > maxStringLength)
					maxStringLength = strlen(buffer_2);
			}
		}

		if(maxStringLength >= SHORT_STRING_MAX_LENGTH)
			fileidx = 1;
		else
			fileidx = 0;
		
		// second pass - write it into files
		buffer_idx = &(buffer[0]);

		for(i = 0 ; i < (language_id+1) ; i++) {
			tab_idx = strchr(buffer_idx, '\t');
			if(tab_idx == NULL && i == language_id) {
				strcpy(buffer_2, buffer_idx);

				if(strlen(buffer_2) > maxStringLength)
					maxStringLength = strlen(buffer_2);

				fprintf(out_fp[i-1][fileidx], "%s", buffer_2);
				continue;
			}
			else {
				strncpy(buffer_2, buffer_idx, tab_idx-buffer_idx);
				buffer_2[tab_idx-buffer_idx] = '\0';
				buffer_idx += ((tab_idx - buffer_idx) +1);
			}

			if(i == 0) {	// definition
				if(fileidx == 0)
					shortCnt++;
				else
					longCnt++;

				fprintf(header_fp[fileidx], "\t%s,\n", buffer_2);

				// add to list table
				char *pivot = strstr(buffer_2, "STRING_");
				if(pivot == NULL) {
					printf("illegal string id exist: %s\n", buffer_2);
					exit(3);
				}
				else
					fprintf(list_fp[fileidx], "%s\n", buffer_2 + strlen("STRING_"));
			}
			else {		// string table
				if(strlen(buffer_2) > maxStringLength)
					maxStringLength = strlen(buffer_2);
				fprintf(out_fp[i-1][fileidx], "%s\n", buffer_2);
			}
		}
	}

	// last pass - combine long string into short strings
	// reset offset to begining
	fseek(header_fp[1], 0L, SEEK_SET);

	while(feof(header_fp[1]) == 0) {
		char tmp[256];
		size_t num;
		num = fread(tmp, sizeof(char), 256, header_fp[1]);
		fwrite(tmp, sizeof(char), num, header_fp[0]);
	}
		
	for(i = 0 ; i < language_id ; i++) {
		fseek(out_fp[i][1], 0L, SEEK_SET);

		while(feof(out_fp[i][1]) == 0) {
			char tmp[256];
			size_t num;
			num = fread(tmp, sizeof(char), 256, out_fp[i][1]);
			fwrite(tmp, sizeof(char), num, out_fp[i][0]);
		}
	}

	fprintf(header_fp[0], "\tSTRING_NUM\n");
	fprintf(header_fp[0], "} ENUM_STRING;\n\n");
	fprintf(header_fp[0], "#endif\n\n");
	fprintf(header_fp[0], "#define LONG_STRING_ID %d\n\n", shortCnt);
	fprintf(header_fp[0], "/* short string count = %d \n", shortCnt);
	fprintf(header_fp[0], "   long string count = %d */\n", longCnt);


	// combine long string id into list table
	fseek(list_fp[1], 0L, SEEK_SET);

	while(feof(list_fp[1]) == 0) {
		char tmp[256];
		size_t num;
		num = fread(tmp, sizeof(char), 256, list_fp[1]);
		fwrite(tmp, sizeof(char), num, list_fp[0]);
	}

	for(i=0;i<language_id;i++) {
		fclose(out_fp[i][0]);
		fclose(out_fp[i][1]);
	}

	fclose(header_fp[0]);
	fclose(header_fp[1]);

	fclose(list_fp[0]);
	fclose(list_fp[1]);

	fclose(fp);
	return 0;

}
